
# q-expansion of newform 8640.2.a.i, downloaded from the LMFDB on 22 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8640
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-1], [-2], [1], [-1], [-1], [4], [-1], [5], [-1], [-6], [0], [7], [-7], [12], [-4], [-10], [-4], [-12], [6], [-15], [2], [-12], [10], [15], [14], [18], [4], [3], [6], [-9], [22], [-6], [13], [1], [17], [-5], [-12], [10], [-4], [4], [22], [-2], [8], [-5], [-22], [16], [24], [20], [-6], [18], [5], [21], [27], [12], [31], [0], [10], [-2], [-16], [2], [-5], [-8], [8], [-24], [-18], [-8], [-16], [18], [17], [12], [-2], [-19], [16], [15], [9], [11], [36], [31], [-33], [-12], [-8], [26], [-16], [12], [6], [28], [18], [-6], [-28], [20], [-4], [-12], [20], [-9], [-7], [28], [1], [-34], [-37], [28], [-38], [32], [22], [32], [42], [15], [-30], [-35], [-30], [15], [9], [38], [8], [-30], [-9], [12], [4], [24], [-42], [-6], [-4], [24], [-10], [-11], [52], [2], [28], [-42], [12], [-51], [-3], [1], [-6], [7], [6], [47], [36], [0], [-10], [38], [20], [-30], [20], [-26], [31], [6], [-22], [21], [53], [18], [-52], [-19], [-45], [56], [-35], [-20], [-14], [17], [-14], [21], [6], [53], [-61], [-33], [55], [-39], [-31], [10], [59], [-10], [-50], [-28], [31], [0], [4], [-50], [-16], [14], [40], [24], [1], [-53], [-32], [-14], [43], [35], [38], [-22], [-10], [54], [44], [-17], [-20], [-10], [-66], [-21], [-46], [16], [51], [-28], [58], [30], [63], [4], [11], [22], [18], [20], [64], [-5], [28], [30], [-46], [-9], [-46], [-70], [9], [-34], [-44], [59], [28], [-60], [-42], [14], [23], [58], [-32], [41], [26], [42], [37], [-26], [60], [36], [75], [-72], [-8], [0], [16], [28], [-58], [-70], [-34], [-32], [-10], [-25], [-62], [19], [45], [52], [-72], [50], [-14], [-39], [-44], [33], [-30], [30], [-28], [-22], [6], [31], [5], [64], [-46], [0], [-9], [-4], [-20], [-11], [66], [43], [-22], [-32], [-8], [4], [-43], [-45], [-39], [39], [-17], [-40], [7], [62], [-16], [-18], [-57], [-42], [-61], [-12], [-15], [60], [50], [-42], [-80], [6], [-83], [-36], [-48], [-24], [67], [-58], [30], [34], [18], [-46], [42], [-11], [23], [-18], [66], [-17], [7], [55], [7], [24], [-68], [-26], [-28], [8], [45], [-76], [-5], [-39], [4], [92], [55], [-60], [34], [-58], [53], [-2], [28], [54], [40], [-6], [-53], [54], [61], [27], [26], [55], [-4], [-5], [-22], [-20], [-84], [-78], [-20], [0], [-31], [10], [-20], [-95], [12], [-5], [25], [19], [58], [52], [-44], [-69], [27], [-80], [-36], [34], [-85], [27], [-10], [97], [66], [-72], [-59], [-81], [76], [-88], [54], [-22], [-35], [49], [48], [-63], [6], [-43], [-40], [-18], [15], [-79], [13], [54], [84], [-77], [-20], [56], [40], [-84], [50], [51], [-36], [-27], [-76], [33], [90], [48], [29], [-18], [52], [-15], [34], [30], [26], [-74], [46], [90], [-46], [-88], [90], [-21], [-75], [62], [-37], [-59], [6], [6], [-72], [-48], [26], [-14], [55], [-27], [4], [-32], [27], [-92], [-17], [-34], [-17], [-56], [76], [-72], [-54], [26], [-67], [82], [91], [-33], [35], [-68], [12], [84], [6], [62], [-24], [3], [-12], [108], [3], [35], [-52], [-32], [69], [44], [20], [86], [17], [-88], [-44], [82], [-68], [40], [-36], [7], [33], [29], [90], [-104], [8], [30], [14], [90], [54]]
